c
c     A set of fortran functions and subroutines
c
c     Author:  P.A. Baedecker, U.S. Geological Survey, Reston, Va. 22092
c
c     Functions      :  ceil, floor, round, length, sigfig
c
c     Subroutines    :  convuppr,convlowr,question,reverse,strdat,translate,
c                       verify
c
c*******************************************************************************
c
c     ceil(x)        :  returns the smallest integer >= x
c
c     floor(x)       :  returns the greatest integer <= x
c
c     round(x,k)     :  returns x rounded to k digits to the right of the
c                       decimal point if k > 0.  Returns x rounded to -k
c                       digits to the left of the decimal point if k < 0.
c
c     sigfig(x,k)    :  rounds x to k significant figures
c
c     length(string) :  returns location of last non-blank character in string
c
c     call convuppr(string) :  converts string to upper case
c
c     call convlowr(string) :  converts string to lower case
c
c     call question(answ) :
c          Answ is a character*3 argument with the value of 'yes', or 'no '.
c          On input it is the default answer to the question posed in the
c          write to terminal statement immediately preceding the call to
c          question.  If a null response is given then the default answer
c          is returned.  The non-null response must be 'yes', 'no ','y  ',
c          or 'n  ' (upper or lower case).
c
c     call reverse(string) : returns string in reverse order
c          (e.g. call reverse(ABCD) yields 'DCBA'
c
c     call strdat(iday,iyr,str) :
c          returns date as a character*8 string (_ddmmmyy)
c          iday is julian data, iyr is year
c
c     call translate(string1,string2,string3) :
c          translates each occurence of a charcter in string3 in string1
c          to the corresponding character in string2
c          (e.g. call translate(12xy89,XY,xy) yields '12XY89'
c
c     call verify(string1,string2,n) :
c          returns the location (n) of the leftmost character in string1
c          not present in string2.  returns 0 if each of the characters
c          in string1 occurs in string2.
c
c*******************************************************************************
c
c     function ceil
c
      real function ceil(x)
      i=int(x)
      b=float(i)
      c=x-b
      if(c.gt.0.0) i=i+1
      ceil=float(i)
      end
c
c     function floor
c
      real function floor(x)
      i=int(x)
      y=i
      if (x.lt.0.0 .and. x.ne.y) i=i-1
      floor=float(i)
      end
c
c     function round
c
      function round (x,k)
      s=0
      if (x.gt.0.0) s=1.0
      if (x.lt.0.0) s=-1.0
      ex=k
      r=s*10.0**(-ex) * floor(abs(x) * 10.0**ex + 0.5)
      round=r
      return
      end
c
c     function sigfig
c
      function sigfig(x,i)
      j=i-1
      if(j.lt.0) then
        sigfig=0.0
        return
      end if
      s=1.0
      if(x.lt.0.0) s=-1.0
      x=abs(x)
      a=alog10(x)
      b=aint(a)
      if(a.lt.0) b=b-1
      b=10.0**b
      c=x/b
      d=round(c,j)
      sigfig=d*b*s
      return
      end 
c
c     function length
c
      integer function length (dummy)
      character*(*) dummy
      character*256 string
      character*1 nul,ht,sp/' '/,ch1
      nul=char(0)
      ht=char(9)
      i=len(dummy)
      length=0
      if(i.le.256) then
        string(1:i)=dummy(1:i)
        do j=i,1,-1
          ch1=string(j:j)
          if(ch1.ne.nul.and.ch1.ne.ht.and.ch1.ne.sp) go to 1
        end do
1       length=j
      end if
      end
	subroutine str$upcase (outstr,instr)
	character*(*) outstr,instr
	outstr=instr
	call convuppr(outstr)
	return
	end
	subroutine str$trim (outstr, instr, nc)
	character*(*) instr,outstr
	outstr=instr
	nc=nblank(outstr)
	outstr=charnb(outstr)
	return
	end
c
c     subroutine convuppr
c          convert string to upper case
c
      subroutine convuppr(dummy)
      character*(*) dummy
      character*256 string,blank
      character*1 cblank(256)
      character*26 upper /'ABCDEFGHIJKLMNOPQRSTUVWXYZ'/
      character*26 lower /'abcdefghijklmnopqrstuvwxyz'/
      data cblank/256*' '/
      equivalence (cblank(1),blank)
      string=blank
      l=length(dummy)
      if(l.eq.0.or.l.gt.256) return
      string(1:l)=dummy(1:l)
      do i=1,l
        do j=1,26
          if(string(i:i).eq.lower(j:j)) string(i:i)=upper(j:j)
        end do
      end do
      dummy(1:l)=string(1:l)
      return
      end
c
c     subroutine convlowr
c        convert to lower case
c
      subroutine convlowr(dummy)
      character*(*) dummy
      character*256 string,blank
      character*1 cblank(256)
      character*26 upper /'ABCDEFGHIJKLMNOPQRSTUVWXYZ'/
      character*26 lower /'abcdefghijklmnopqrstuvwxyz'/
      data cblank/256*' '/
      equivalence (cblank(1),blank)
      string=blank
      l=length(dummy)
      if(l.eq.0.or.l.gt.256) return
      string(1:l)=dummy(1:l)
      do i=1,l
        do j=1,26
          if(string(i:i).eq.upper(j:j)) string(i:i)=lower(j:j)
        end do
      end do
      dummy(1:l)=string(1:l)
      return
      end
c
c     subroutine translate
c
      subroutine translate(dummy,strng1,strng2)
	integer i,j,l,m,n
      character*(*) dummy,strng1,strng2
      character*256 string,blank,strgto,strgfm
      character*1 cblank(256)
      data cblank/256*' '/
      equivalence (cblank(1),blank)
      string=blank
      strgto=blank
      strgfm=blank
      l=length(dummy)
      if(l.eq.0.or.l.gt.256) return
      m=length(strng1)
      if(m.gt.256) return
      if(m.gt.0) strgto(1:m)=strng1(1:m)
      n=length(strng2)
      if(n.gt.256) return
      if(n.gt.0) strgfm(1:n)=strng2(1:n)
      if(m.lt.n) m=n
      if(n.lt.m) n=m
      string(1:l)=dummy(1:l)
      do i=1,l
        do j=1,n
          if(string(i:i).eq.strgfm(j:j)) string(i:i)=strgto(j:j)
        end do
      end do
      dummy(1:l)=string(1:l)
      return
      end
c
c     subroutine verify
c
      subroutine verify(dummy1,dummy2,n)
      integer i,j,k,l,m,n
      character*(*) dummy1,dummy2
      character*256 strng1,strng2
      k=len(dummy1)
      l=len(dummy2)
      strng1(1:k)=dummy1(1:k)
      strng2(1:l)=dummy2(1:l)
      m1=length(dummy1)
      n=0
      do i=1,m1
        m=0
        do j=1,l
          if(strng1(i:i).eq.strng2(j:j)) m=j
        end do
        if(m.eq.0) then
          n=i
          return
        end if
      end do
      return
      end
c
c     subroutine reverse
c
      subroutine reverse (dummy,icode)
      character*(*) dummy
      character*1 ch1
      character*256 string
      icode=0
      i=len(dummy)
      if(i.le.256) then
        string(1:i)=dummy(1:i)
        j=length(string)
        k=j+1
        j=j/2
        do 1 m=1,j
          k=k-1
          ch1=string(m:m)
          string(m:m)=string(k:k)
          string(k:k)=ch1
1       continue
        dummy(1:i)=string(1:i)
      else
        icode=1
      end if
      return
      end
c
c     subroutine question
c
      subroutine question(rsp)
      character*3 rsp,deflt
1     format(a3)
2     format('$',a)   
      go to 4
3     continue
c      rewind(4)  ! IBM version
      go to 6
4     deflt=rsp
5     rsp='   '
	read(4,1,end=3) rsp
6     if(rsp.eq.'   ') rsp=deflt
      call convlowr(rsp,3)
      nch=length(rsp)
      if(nch.eq.1) then
        if (rsp(1:1).eq.'y') rsp='yes'
        if (rsp(1:1).eq.'n') rsp='no '
      end if
      if (rsp.eq.'yes'.or.rsp.eq.'no ') return
	write(4,2) 'Answer yes(y) or no(n): '
      go to 5
      end
c
c     subroutine strdat
c
      subroutine strdat(iday,iyr,str)
      integer ndays(12)
      character*2 ch2
      character*3 mon(12),ch3
      character*8 str
      data mon/'JAN','FEB','MAR','APR','MAY','JUN',
     &         'JUL','AUG','SEP','OCT','NOV','DEC'/
      data ndays/31,28,31,30,31,30,31,31,30,31,30,31/
1     format(i2)
2     format(i3)
      write(ch2,1) iyr
      a=iyr-80
      a=a/4.0
      b=aint(a)
      a=a-b
      if(a.eq.0.0) ndays(2)=29
      n=0
      do i=1,12
        m=n
        n=n+ndays(i)
        if(iday.le.n) then
          j=iday-m
          write(ch3,2) j
          str=ch3//mon(i)//ch2
          ndays(2)=28
          return
        end if
      end do
      end

